#!/usr/bin/env python3
VERSION="1.3.0"
import sys
from time import sleep
import os
import types
import ctypes
import urllib.request
import multiprocessing
import queue
from PyQt5.QtCore import Qt
from PyQt5.QtGui import *
from PyQt5.QtWidgets import QGridLayout, QLabel, QLineEdit, QMessageBox, QColorDialog
from PyQt5.QtWidgets import QTextEdit, QWidget, QMainWindow, QApplication, QListWidgetItem, QTableWidgetItem

from . import hue_gui 
from . import hue
#import hue_gui # REMEMBER TO CHANGE BACK
#import hue

import serial
from serial.tools import list_ports

from . import webcolors
#import webcolors

audio_lock = False

def is_admin():
    if os.name == 'nt':
        # WARNING: requires Windows XP SP2 or higher!
        try:
            return ctypes.windll.shell32.IsUserAnAdmin()
        except:
            return False
    else:
        return True

def runAsAdmin():

    if os.name != 'nt':
        return
    else:
        ctypes.windll.shell32.ShellExecuteW(None, "runas", sys.executable, "", None, 1)

def main():
    #if os.geteuid() != 0:
    #    sys.exit("You need to have root privileges to run this script.\nPlease try again, this time using 'sudo'.")
    #if not is_admin():
    #    runAsAdmin()
    app = QApplication(sys.argv)
    form = MainWindow()
    form.show()
    sys.exit(app.exec_())

def closest_colour(requested_colour):
    min_colours = {}
    for key, name in webcolors.css3_hex_to_names.items():
        r_c, g_c, b_c = webcolors.hex_to_rgb(key)
        rd = (r_c - requested_colour[0]) ** 2
        gd = (g_c - requested_colour[1]) ** 2
        bd = (b_c - requested_colour[2]) ** 2
        min_colours[(rd + gd + bd)] = name
    return min_colours[min(min_colours.keys())]

def get_colour_name(requested_colour):
    try:
        closest_name = actual_name = webcolors.rgb_to_name(requested_colour)
    except ValueError:
        closest_name = closest_colour(requested_colour)
        actual_name = None
    return actual_name, closest_name

def find_between( s, first, last ):
    try:
        start = s.index( first ) + len( first )
        end = s.index( last, start )
        return s[start:end]
    except ValueError:
        return ""

def pick(n):
    c = QColorDialog.getColor()
    if c.isValid():
        return c.name()[1:].upper()
def versiontuple(v):
    return tuple(map(int, (v.split("."))))

class MainWindow(QMainWindow, hue_gui.Ui_MainWindow):
    def __init__(self):
        super(MainWindow, self).__init__()
        self.setupUi(self)

        self.indexApply = {
            0: self.fixedApply,
            1: self.breathingApply,
            2: self.fadingApply,
            3: self.marqueeApply,
            4: self.coverMarqueeApply,
            5: self.pulseApply,
            6: self.spectrumApply,
            7: self.alternatingApply,
            8: self.candleApply,
            9: self.wingsApply,
            10: self.audioLevelApply,
            11: self.customApply,
            12: self.profileApply
            }

        self.fixedAdd.clicked.connect(self.fixedAddFunc)
        self.fixedDelete.clicked.connect(self.fixedDeleteFunc)
        self.breathingAdd.clicked.connect(self.breathingAddFunc)
        self.breathingDelete.clicked.connect(self.breathingDeleteFunc)
        self.fadingAdd.clicked.connect(self.fadingAddFunc)
        self.fadingDelete.clicked.connect(self.fadingDeleteFunc)
        self.marqueeAdd.clicked.connect(self.marqueeAddFunc)
        self.marqueeDelete.clicked.connect(self.marqueeDeleteFunc)
        self.coverMarqueeAdd.clicked.connect(self.coverMarqueeAddFunc)
        self.coverMarqueeDelete.clicked.connect(self.coverMarqueeDeleteFunc)
        self.pulseAdd.clicked.connect(self.pulseAddFunc)
        self.pulseDelete.clicked.connect(self.pulseDeleteFunc)
        self.alternatingAdd.clicked.connect(self.alternatingAddFunc)
        self.alternatingDelete.clicked.connect(self.alternatingDeleteFunc)
        self.candleAdd.clicked.connect(self.candleAddFunc)
        self.candleDelete.clicked.connect(self.candleDeleteFunc)
        self.wingsAdd.clicked.connect(self.wingsAddFunc)
        self.wingsDelete.clicked.connect(self.wingsDeleteFunc)
        self.audioLevelAdd.clicked.connect(self.audioLevelAddFunc)
        self.audioLevelDelete.clicked.connect(self.audioLevelDeleteFunc)
        self.customEdit.clicked.connect(self.customEditFunc)
        self.profileAdd.clicked.connect(self.profileAddFunc)
        self.profileDelete.clicked.connect(self.profileDeleteFunc)
        self.profileRefresh.clicked.connect(self.profileListFunc)
        self.applyBtn.clicked.connect(self.applyFunc)

        self.populateCustom()

        if os.name == 'nt':
            self.portTxt.setText('COM3')
        self.update()
        port = self.get_port()
        if port:
            self.portTxt.setText(self.get_port())
        else:
            self.error("No Hue+ found.")

        self.profileListFunc()
        try:
            with serial.Serial(self.portTxt.text(), 256000) as ser:
                hue.write_previous(ser)
        except serial.serialutil.SerialException:
            self.error("Serial port is invalid. Try /dev/ttyACM0 for Linux or COM3 or COM4 for Windows")

    def closeEvent(self, event):
        self.checkAudio()
        event.accept()

    def error(self, message):
        msg = QMessageBox()
        msg.setText(message)
        msg.setStandardButtons(QMessageBox.Ok)
        out = msg.exec_()

    def get_port(self):
        ports = []
        for port in list_ports.comports():
            if 'MCP2200' in port[1] or 'USB Serial Device' in port[1]:
                ports.append(port[0])
        if ports:
            return ports[0]
        else:
            return None

    def checkAudio(self):
        global audio_lock
        global q
        global audioThread
        if audio_lock:
            audioThread.terminate()
            audio_lock = False
            sleep(0.1)

    def update(self):
        with urllib.request.urlopen('https://raw.githubusercontent.com/kusti8/hue-plus/master/version') as response:
            version_new = response.read().strip()
            if versiontuple(version_new.decode()) > versiontuple(VERSION):
                self.error("There is a new update available. Download it from https://github.com/kusti8/hue-plus")

    def getChannel(self):
        if self.channel1Check.isChecked() and self.channel2Check.isChecked():
            return 0
        elif self.channel1Check.isChecked():
            return 1
        elif self.channel2Check.isChecked():
            return 2
        else:
            return None

    def getColors(self, modeList):
        colors = []
        for i in range(modeList.count()):
            colors.append(find_between(modeList.item(i).text(), '#', ')').upper())
        if modeList.count() == 0:
            self.error("Must have at least one color")
            return ['FF0000']
        return colors

    ## Fixed
    def fixedAddFunc(self):
        if self.fixedList.count() == 1:
            self.error("Fixed cannot have more than one color")
        else:
            hex_color = pick("Color")
            if hex_color is None:
                return
            color = "#" + hex_color.lower()
            actual, closest = get_colour_name(webcolors.hex_to_rgb(color))
            if not actual:
                actual = closest
            self.fixedList.addItem(QListWidgetItem(actual + "(" + color + ")"))

    def fixedDeleteFunc(self):
        self.fixedList.takeItem(self.fixedList.currentRow())

    def fixedApply(self):
        self.checkAudio()
        try:
            with serial.Serial(self.portTxt.text(), 256000) as ser:
                print("Applying")
                if self.getChannel() == None:
                    hue.power(ser, 0, "off")
                else:
                    hue.fixed(ser, 0, self.getChannel(), self.getColors(self.fixedList)[0])
        except serial.serialutil.SerialException:
            self.error("Serial port is invalid. Try /dev/ttyACM0 for Linux or COM3 or COM4 for Windows")

    ## Breathing
    def breathingAddFunc(self):
        color = "#" + pick("Color").lower()
        actual, closest = get_colour_name(webcolors.hex_to_rgb(color))
        if not actual:
            actual = closest
        self.breathingList.addItem(QListWidgetItem(actual + "(" + color + ")"))

    def breathingDeleteFunc(self):
        self.breathingList.takeItem(self.breathingList.currentRow())

    def breathingApply(self):
        self.checkAudio()
        try:
            with serial.Serial(self.portTxt.text(), 256000) as ser:
                if self.getChannel() == None:
                    hue.power(ser, 0, "off")
                else:
                    speed = self.breathingSpeed.value()
                    hue.breathing(ser, 0, self.getChannel(), self.getColors(self.breathingList), speed)
        except serial.serialutil.SerialException:
            self.error("Serial port is invalid. Try /dev/ttyACM0 for Linux or COM3 or COM4 for Windows")

    ## Fading
    def fadingAddFunc(self):
        hex_color = pick("Color")
        if hex_color is None:
            return
        color = "#" + hex_color.lower()
        actual, closest = get_colour_name(webcolors.hex_to_rgb(color))
        if not actual:
            actual = closest
        self.fadingList.addItem(QListWidgetItem(actual + "(" + color + ")"))

    def fadingDeleteFunc(self):
        self.fadingList.takeItem(self.fadingList.currentRow())

    def fadingApply(self):
        self.checkAudio()
        try:
            with serial.Serial(self.portTxt.text(), 256000) as ser:
                if self.getChannel() == None:
                    hue.power(ser, 0, "off")
                else:
                    speed = self.fadingSpeed.value()
                    hue.fading(ser, 0, self.getChannel(), self.getColors(self.fadingList), speed)
        except serial.serialutil.SerialException:
            self.error("Serial port is invalid. Try /dev/ttyACM0 for Linux or COM3 or COM4 for Windows")

    ## Marquee
    def marqueeAddFunc(self):
        if self.marqueeList.count() == 1:
            self.error("Marquee cannot have more than one color")
        else:
            hex_color = pick("Color")
            if hex_color is None:
                return
            color = "#" + hex_color.lower()
            actual, closest = get_colour_name(webcolors.hex_to_rgb(color))
            if not actual:
                actual = closest
            self.marqueeList.addItem(QListWidgetItem(actual + "(" + color + ")"))

    def marqueeDeleteFunc(self):
        self.marqueeList.takeItem(self.marqueeList.currentRow())

    def marqueeApply(self):
        self.checkAudio()
        try:
            with serial.Serial(self.portTxt.text(), 256000) as ser:
                if self.getChannel() == None:
                    hue.power(ser, 0, "off")
                else:
                    speed = self.marqueeSpeed.value()
                    size = self.marqueeSize.value()
                    direction = 0 if self.marqueeBackwards.isChecked() else 0
                    hue.marquee(ser, 0, self.getChannel(), self.getColors(self.marqueeList)[0], speed, size, direction)
        except serial.serialutil.SerialException:
            self.error("Serial port is invalid. Try /dev/ttyACM0 for Linux or COM3 or COM4 for Windows")

    ## coverMarquee
    def coverMarqueeAddFunc(self):
        hex_color = pick("Color")
        if hex_color is None:
            return
        color = "#" + hex_color.lower()
        actual, closest = get_colour_name(webcolors.hex_to_rgb(color))
        if not actual:
            actual = closest
        self.coverMarqueeList.addItem(QListWidgetItem(actual + "(" + color + ")"))

    def coverMarqueeDeleteFunc(self):
        self.coverMarqueeList.takeItem(self.coverMarqueeList.currentRow())

    def coverMarqueeApply(self):
        self.checkAudio()
        try:
            with serial.Serial(self.portTxt.text(), 256000) as ser:
                if self.getChannel() == None:
                    hue.power(ser, 0, "off")
                else:
                    speed = self.coverMarqueeSpeed.value()
                    direction = 0 if self.coverMarqueeBackwards.isChecked() else 0
                    hue.cover_marquee(ser, 0, self.getChannel(), self.getColors(self.coverMarqueeList), speed, direction)
        except serial.serialutil.SerialException:
            self.error("Serial port is invalid. Try /dev/ttyACM0 for Linux or COM3 or COM4 for Windows")

    ## pulse
    def pulseAddFunc(self):
        color = "#" + pick("Color").lower()
        actual, closest = get_colour_name(webcolors.hex_to_rgb(color))
        if not actual:
            actual = closest
        self.pulseList.addItem(QListWidgetItem(actual + "(" + color + ")"))

    def pulseDeleteFunc(self):
        self.pulseList.takeItem(self.pulseList.currentRow())

    def pulseApply(self):
        self.checkAudio()
        try:
            with serial.Serial(self.portTxt.text(), 256000) as ser:
                if self.getChannel() == None:
                    hue.power(ser, 0, "off")
                else:
                    speed = self.pulseSpeed.value()
                    hue.pulse(ser, 0, self.getChannel(), self.getColors(self.pulseList), speed)
        except serial.serialutil.SerialException:
            self.error("Serial port is invalid. Try /dev/ttyACM0 for Linux or COM3 or COM4 for Windows")

    ## spectrum
    def spectrumApply(self):
        self.checkAudio()
        try:
            with serial.Serial(self.portTxt.text(), 256000) as ser:
                if self.getChannel() == None:
                    hue.power(ser, 0, "off")
                else:
                    speed = self.spectrumSpeed.value()
                    direction = 1 if self.spectrumBackwards.isChecked() else 0
                    hue.spectrum(ser, self.getChannel(), speed, direction)
        except serial.serialutil.SerialException:
            self.error("Serial port is invalid. Try /dev/ttyACM0 for Linux or COM3 or COM4 for Windows")

    ## alternating
    def alternatingAddFunc(self):
        if self.alternatingList.count() == 2:
            self.error("Alternating cannot have more than two colors")
        else:
            hex_color = pick("Color")
            if hex_color is None:
                return
            color = "#" + hex_color.lower()
            actual, closest = get_colour_name(webcolors.hex_to_rgb(color))
            if not actual:
                actual = closest
            self.alternatingList.addItem(QListWidgetItem(actual + "(" + color + ")"))

    def alternatingDeleteFunc(self):
        self.alternatingList.takeItem(self.alternatingList.currentRow())

    def alternatingApply(self):
        self.checkAudio()
        try:
            if self.alternatingList.count() != 2:
                self.error("Alternating must have two colors")
            else:
                with serial.Serial(self.portTxt.text(), 256000) as ser:
                    if self.getChannel() == None:
                        hue.power(ser, 0, "off")
                    else:
                        speed = self.alternatingSpeed.value()
                        size = self.alternatingSize.value()
                        direction = 1 if self.alternatingBackwards.isChecked() else 0
                        moving = self.alternatingMoving.isChecked()
                        hue.alternating(ser, 0, self.getChannel(), self.getColors(self.alternatingList), speed, size, moving, direction)
        except serial.serialutil.SerialException:
            self.error("Serial port is invalid. Try /dev/ttyACM0 for Linux or COM3 or COM4 for Windows")
    ## candle
    def candleAddFunc(self):
        if self.candleList.count() == 1:
            self.error("Candle cannot have more than 1 color")
        else:
            hex_color = pick("Color")
            if hex_color is None:
                return
            color = "#" + hex_color.lower()
            actual, closest = get_colour_name(webcolors.hex_to_rgb(color))
            if not actual:
                actual = closest
            self.candleList.addItem(QListWidgetItem(actual + "(" + color + ")"))

    def candleDeleteFunc(self):
        self.candleList.takeItem(self.candleList.currentRow())

    def candleApply(self):
        self.checkAudio()
        try:
            with serial.Serial(self.portTxt.text(), 256000) as ser:
                if self.getChannel() == None:
                    hue.power(ser, 0, "off")
                else:
                    hue.candlelight(ser, 0, self.getChannel(), self.getColors(self.candleList)[0])
        except serial.serialutil.SerialException:
            self.error("Serial port is invalid. Try /dev/ttyACM0 for Linux or COM3 or COM4 for Windows")

    ## wings
    def wingsAddFunc(self):
        if self.wingsList.count() == 1:
            self.error("Wings cannot have more than 1 color")
        else:
            hex_color = pick("Color")
            if hex_color is None:
                return
            color = "#" + hex_color.lower()
            actual, closest = get_colour_name(webcolors.hex_to_rgb(color))
            if not actual:
                actual = closest
            self.wingsList.addItem(QListWidgetItem(actual + "(" + color + ")"))

    def wingsDeleteFunc(self):
        self.wingsList.takeItem(self.wingsList.currentRow())

    def wingsApply(self):
        self.checkAudio()
        try:
            with serial.Serial(self.portTxt.text(), 256000) as ser:
                if self.getChannel() == None:
                    hue.power(ser, 0, "off")
                else:
                    speed = self.wingsSpeed.value()
                    hue.wings(ser, 0, self.getChannel(), self.getColors(self.wingsList)[0], speed)
        except serial.serialutil.SerialException:
            self.error("Serial port is invalid. Try /dev/ttyACM0 for Linux or COM3 or COM4 for Windows")

    ## audio_level
    def audioLevelAddFunc(self):
        hex_color = pick("Color")
        if hex_color is None:
            return
        color = "#" + hex_color.lower()
        actual, closest = get_colour_name(webcolors.hex_to_rgb(color))
        if not actual:
            actual = closest
        self.audioLevelList.addItem(QListWidgetItem(actual + "(" + color + ")"))

    def audioLevelDeleteFunc(self):
        self.audioLevelList.takeItem(self.audioLevelList.currentRow())

    def audioLevelApply(self):
        global audio_lock
        global audioThread
        global q
        if os.name == 'nt':
            self.error("To enable audio mode on Windows, right click on the audio icon, go to recording devices, right click and select show disabled devices, and right click on stereo mix and click enable.")
        self.checkAudio()
        try:
            ser = serial.Serial(self.portTxt.text(), 256000)
            if self.getChannel() == None:
                hue.power(ser, 0, "off")
            else:
                tolerance = float(self.audioLevelTolerance.value())
                smooth = int(self.audioLevelSmooth.value())
                audioThread = multiprocessing.Process(target=hue.audio_level, args=(self.portTxt.text(), 0, self.getChannel(), self.getColors(self.audioLevelList), tolerance, smooth))
                audio_lock=True
                audioThread.start()
        except serial.serialutil.SerialException:
            self.error("Serial port is invalid. Try /dev/ttyACM0 for Linux or COM3 or COM4 for Windows")

    ## profile
    def profileAddFunc(self):
        hue.profile_add(self.profileName.text())
        self.profileList.addItem(QListWidgetItem(self.profileName.text()))

    def profileDeleteFunc(self):
        hue.profile_rm(self.profileList.currentItem().text())
        self.audioLevelList.takeItem(self.profileList.currentRow())
        self.profileListFunc()

    def profileApply(self):
        self.checkAudio()
        try:
            with serial.Serial(self.portTxt.text(), 256000) as ser:
                if self.getChannel() == None:
                    hue.power(ser, 0, "off")
                else:
                    hue.profile_apply(ser, self.profileList.currentItem().text())
        except serial.serialutil.SerialException:
            self.error("Serial port is invalid. Try /dev/ttyACM0 for Linux or COM3 or COM4 for Windows")

    def profileListFunc(self):
        self.profileList.clear()
        if hue.profile_list():
            for p in hue.profile_list():
                self.profileList.addItem(QListWidgetItem(p))

    def applyFunc(self):
        self.indexApply[self.presetModeWidget.currentIndex()]()

    # custom
    def populateCustom(self):
        actual, closest = get_colour_name(webcolors.hex_to_rgb('#FFFFFF'))
        if not actual:
            actual = closest
        for i in range(40):
            self.customTable.setItem(i, 0, QTableWidgetItem(str(i+1)))
            self.customTable.setItem(i, 1, QTableWidgetItem(actual + '(#FFFFFF)'))

    def customEditFunc(self):
        hex_color = pick("Color")
        if hex_color is None:
            return
        color = "#" + hex_color.lower()
        actual, closest = get_colour_name(webcolors.hex_to_rgb(color))
        if not actual:
            actual = closest
        
        for widgetItem in self.customTable.selectedItems():
            if widgetItem.column() != 0:
                widgetItem.setText(actual + "(" + color + ")")
                widgetItem.setBackground(QColor(*webcolors.hex_to_rgb(color)))

    def customGetColors(self):
        colors = []
        for i in range(40):
            colors.append(find_between(self.customTable.item(i, 1).text(), '#', ')').upper())
        return colors

    def customApply(self):
        self.checkAudio()
        try:
            with serial.Serial(self.portTxt.text(), 256000) as ser:
                if self.getChannel() == None:
                    hue.power(ser, 0, "off")
                else:
                    speed = self.customSpeed.value()
                    hue.custom(ser, 0, self.getChannel(), self.customGetColors(), self.customMode.currentText().lower(), speed)
        except serial.serialutil.SerialException:
            self.error("Serial port is invalid. Try /dev/ttyACM0 for Linux or COM3 or COM4 for Windows")

if __name__ == '__main__':
    main()



#!/usr/bin/env python3
import serial
import time
from time import sleep
import sys
import inspect
import argparse
import os
from . import picker
from . import previous
#import picker
#import previous
import sys
import struct
import math
import colorsys

# Exceptions
class InvalidCommand(Exception):
    pass

class InvalidUser(Exception):
    pass

def main():
    #if os.geteuid() != 0:
    #    sys.exit("You need to have root privileges to run this script.\nPlease try again, this time using 'sudo'.")

    parser = argparse.ArgumentParser(description="Change NZXT Hue+ LEDs")
    parser.add_argument("-p", "--port", default="/dev/ttyACM0", type=str, help="The port, defaults to /dev/ttyACM0. On Windows, it is usually COM3 or COM4")
    parser.add_argument("-c", "--channel", type=int, default=0, help="The channel, defaults to 0 (BOTH)")
    parser.add_argument("-g", "--gui", type=int, default=0, help="How many colors of GUI picker")
    subparsers = parser.add_subparsers(help="The type of color (fixed, breathing)", dest='command')

    parser_fixed = subparsers.add_parser('fixed', help="One single fixed color")
    parser_fixed.add_argument("color", type=str, help="Color in hex")

    parser_breathing = subparsers.add_parser('breathing', help="Breathing through a set of colors")
    parser_breathing.add_argument("speed", type=int, help="Speed from 0(Slowest) to 4(Fastest)")
    parser_breathing.add_argument("colors", type=str, nargs='+', help="Color(s) in hex")

    parser_fading = subparsers.add_parser('fading', help="Fading through a set of colors")
    parser_fading.add_argument("speed", type=int, help="Speed from 0(Slowest) to 4(Fastest)")
    parser_fading.add_argument("colors", type=str, nargs='+', help="Color(s) in hex")

    parser_marquee = subparsers.add_parser('marquee', help="A strip of color running")
    parser_marquee.add_argument("speed", type=int, help="Speed from 0(Slowest) to 4(Fastest)")
    parser_marquee.add_argument("-b", "--backwards", action="store_const", const=0, default=0, help="Enable going backwards")
    parser_marquee.add_argument("size", type=int, help="The size of the group of runners (0=3, 1=4, 2=5, 3=6)")
    parser_marquee.add_argument("color", type=str, help="Foreground color in hex")

    parser_cover_marquee = subparsers.add_parser('cover_marquee', help="A strip of color running (multiple colors)")
    parser_cover_marquee.add_argument("speed", type=int, help="Speed from 0(Slowest) to 4(Fastest)")
    parser_cover_marquee.add_argument("-b", "--backwards", action="store_const", const=0, default=0, help="Enable going backwards")
    parser_cover_marquee.add_argument("colors", type=str, nargs='+', help="Colors in hex")

    parser_pulse = subparsers.add_parser('pulse', help="Pulsing through a set of colors")
    parser_pulse.add_argument("speed", type=int, help="Speed from 0(Slowest) to 4(Fastest)")
    parser_pulse.add_argument("colors", type=str, nargs='+', help="Color(s) in hex")

    parser_spectrum = subparsers.add_parser('spectrum', help="Pulsing through a set of colors")
    parser_spectrum.add_argument("speed", type=int, help="Speed from 0(Slowest) to 4(Fastest)")
    parser_spectrum.add_argument("-b", "--backwards", action="store_const", const=1, default=0, help="Enable going backwards")

    parser_alternating = subparsers.add_parser('alternating', help="Two alternating colors")
    parser_alternating.add_argument("speed", type=int, help="Speed from 0(Slowest) to 4(Fastest)")
    parser_alternating.add_argument("-m", "--moving", action="store_true", help="Enable movement")
    parser_alternating.add_argument("-b", "--backwards", action="store_const", const=1, default=0, help="Enable going backwards (requires movement)")
    parser_alternating.add_argument("size", type=int, help="The size of the group of runners (0=2, 1=3, 2=5, 3=10)")
    parser_alternating.add_argument("colors", type=str, nargs=2, help="First and second colors in hex")

    parser_candlelight = subparsers.add_parser('candlelight', help="A flickering color")
    parser_candlelight.add_argument("color", type=str, help="Color in hex")

    parser_wings = subparsers.add_parser('wings', help="Strips of light meeting in the center")
    parser_wings.add_argument("speed", type=int, help="Speed from 0(Slowest) to 4(Fastest)")
    parser_wings.add_argument("color", type=str, help="Color in hex")

    parser_audio_level = subparsers.add_parser('audio_level', help="Light syncronized to music levels")
    parser_audio_level.add_argument("tolerance", type=float, help="The maximum audio level ie. when the audio is as loud as tolerance, all LEDs will be lit")
    parser_audio_level.add_argument("refresh", type=int, help="The speed of refreshing the LEDs (usually 5 is a good number)")
    parser_audio_level.add_argument("colors", type=str, nargs='+', help="Colors in hex, starting from lowest volume to highest")

    parser_custom = subparsers.add_parser('custom', help="Custom light for each individual LED")
    parser_custom.add_argument("mode", type=str, help="A choice of modes for custom of fixed, breathing, or wave")
    parser_custom.add_argument("speed", type=int, help="Speed from 0(Slowest) to 4(Fastest)")
    parser_custom.add_argument("colors", type=str, nargs='+', help="Colors in hex, starting from lowest volume to highest")

    parser_power = subparsers.add_parser('power', help="Control power to the channels")
    parser_power.add_argument("state", type=str, help="State (on/off)")

    parser_profile = subparsers.add_parser('profile', help="Add or remove or apply profiles")
    subparsers_profile = parser_profile.add_subparsers(help="The type of profile action", dest='profile_command')

    parser_profile_add = subparsers_profile.add_parser('add', help="Add a profile of the current")
    parser_profile_add.add_argument("name", type=str, help="Name of the profile")

    parser_profile_remove = subparsers_profile.add_parser('rm', help="Remove a profile of the current")
    parser_profile_remove.add_argument("name", type=str, help="Name of the profile")

    parser_profile_list = subparsers_profile.add_parser('list', help="List the profiles")

    parser_profile_apply = subparsers_profile.add_parser('apply', help="Apply a profile")
    parser_profile_apply.add_argument("name", type=str, help="Name of the profile")

    args = parser.parse_args()

    ser = serial.Serial(args.port, 256000)

    if args.command == "fixed":
        fixed(ser, args.gui, args.channel, args.color)
    elif args.command == 'breathing':
        breathing(ser, args.gui, args.channel, args.colors, args.speed)
    elif args.command == 'fading':
        fading(ser, args.gui, args.channel, args.colors, args.speed)
    elif args.command == 'marquee':
        marquee(ser, args.gui, args.channel, args.color, args.speed, args.size, args.backwards)
    elif args.command == 'cover_marquee':
        cover_marquee(ser, args.gui, args.channel, args.colors, args.speed, args.backwards)
    elif args.command == 'pulse':
        pulse(ser, args.gui, args.channel, args.colors, args.speed)
    elif args.command == 'spectrum':
        spectrum(ser, args.channel, args.speed, args.backwards)
    elif args.command == 'alternating':
        alternating(ser, args.gui, args.channel, args.colors, args.speed, args.size, args.moving, args.backwards)
    elif args.command == 'candlelight':
        candlelight(ser, args.gui, args.channel, args.color)
    elif args.command == 'wings':
        wings(ser, args.gui, args.channel, args.color, args.speed)
    elif args.command == 'audio_level':
        audio_level(ser, args.gui, args.channel, args.colors, args.tolerance, args.refresh)
    elif args.command == 'custom':
        custom(ser, args.gui, args.channel, args.colors, args.mode, args.speed)
    elif args.command == 'power':
        power(ser, args.channel, args.state)
    elif args.command == 'profile':
        if args.profile_command == 'add':
            profile_add(args.name)
        elif args.profile_command == 'rm':
            profile_rm(args.name)
        elif args.profile_command == 'apply':
            profile_apply(ser, args.name)
        elif args.profile_command == 'list':
            profile_list()
        else:
            raise InvalidCommand("No such profile command")
    else:
        raise InvalidCommand("No such mode")

def write_audio(ser, channel, colors, tolerance, value, strips):
    try:
        if value >= tolerance: # Prevent index out of range
            value = tolerance
        elif value < 0.0:
            value = 0.0
        normal = (value/tolerance)*int(strips[channel-1]*10)
        normal_color = (value/tolerance)*len(colors)
        size = int(strips[channel-1]*10/len(colors))
        value = normal_color*size-int(normal_color*size) # Get value fraction
        dis = colors[:int(normal_color)]
        display = []
        for color in dis:
            for i in range(int(size)):
                display.append(color)
        for i in range(int(value*size)):
            display.append(colors[int(normal_color)])
        rgb = colors[int(normal_color)]
        rgb = (int(rgb[:2], 16)/255.0, int(rgb[2:4], 16)/255.0, int(rgb[4:], 16)/255.0)
        h, s, v = colorsys.rgb_to_hsv(*rgb)
        v = value*v # Change value of final color
        r,g,b = colorsys.hsv_to_rgb(h, s, v)
        r = int(r*255)
        g = int(g*255)
        b = int(b*255)
        hexcolor = '%02x%02x%02x' % (r,g,b) # Back to html notation
        display.append(hexcolor.upper())
    except KeyboardInterrupt:
        raise
    except:
        return
    command = create_custom(ser, channel, display, "audio", 0, 0, 0, 0, strips)
    outputs = previous.get_colors(channel, command)
    write(ser, outputs)

def audio_level(ser, gui, channel, colors, tolerance, smooth):
    try:
        if os.geteuid() == 0:
            raise InvalidUser("Audio won't work with root. Login as a normal user, add your user to the group of /dev/ttyACM0 and retry without root")
    except:
        pass

    ser_new = False
    if not inspect.isclass(ser):
        ser_new = True
        ser = serial.Serial(ser, 256000)
    
    if 1 <= gui <= 8:
        colors = []
        for i in range(gui):
            colors.append(picker.pick("Color "+str(i+1) + " of "+str(gui)))
    strips = [strips_info(ser, 1), strips_info(ser, 2)]
    if channel == 0:
        channel = [1, 2]
    else:
        channel = [channel]
    init(ser)
    import pyaudio
    import wave

    chunk = 2048
    FORMAT = pyaudio.paInt16
    CHANNELS = 1
    RATE = 44100
    RECORD_SECONDS = 500000
    WAVE_OUTPUT_FILENAME = "output.wav"
    p = pyaudio.PyAudio()
    if os.name == 'nt':
        chunk = 1024 # Windows doesn't like 2048 for some reason
        index = p.get_default_input_device_info()['index']
        for i in range(p.get_device_count()):
            if 'Stereo Mix' in p.get_device_info_by_index(i)['name']:
                index = i
        stream = p.open(format = FORMAT,
                        channels = CHANNELS,
                        rate = RATE,
                        input = True,
                        output = True,
                        frames_per_buffer = chunk,
                        input_device_index=index)
    else:
        stream = p.open(format = FORMAT,
                        channels = CHANNELS,
                        rate = RATE,
                        input = True,
                        output = True,
                        frames_per_buffer = chunk)
    alls = []
    s = []
    try:
        while True:
            try:
                data = stream.read(chunk)
            except IOError:
                continue
            alls.append(data)
            if len(alls)>1:
                data = b''.join(alls)
                wf = wave.open(WAVE_OUTPUT_FILENAME, 'wb')
                wf.setnchannels(CHANNELS)
                wf.setsampwidth(p.get_sample_size(FORMAT))
                wf.setframerate(RATE)
                wf.writeframes(data)
                wf.close()
                w = wave.open(WAVE_OUTPUT_FILENAME, 'rb')
                summ = 0
                value = 1
                delta = 1
                amps = [ ]
                for i in range(0, w.getnframes()):
                    data = struct.unpack('<h', w.readframes(1))
                    summ += (data[0]*data[0]) / 2
                    if (i != 0 and (i % 1470) == 0):
                        value = int(math.sqrt(summ / 1470.0) / 10)
                        amps.append(value - delta)
                        summ = 0
                        if delta == 0:
                            tarW=0.0
                        else:
                            tarW=float(amps[0]*1.0/delta/100)
                        s.append(tarW)
                        if len(s) >= smooth:
                            out = sum(s)/len(s)
                            for c in channel:
                                write_audio(ser, c, colors, tolerance, out, strips)
                            s = []
                        delta = value
                alls=[]
    except:
        stream.close()
        p.terminate()
        os.remove(WAVE_OUTPUT_FILENAME)
        raise
    if ser_new:
        ser.close()

def create_custom(ser, channel, colors, mode, direction, option, group, speed, strips):
    if colors == None:
        colors = []

    commands = []
    channel_commands = []
    modes = {
        "audio": 14,
        "fixed": 0,
        "breathing": 7,
        "wave": 13
    }
    if modes[mode] != 14: # Audio flickers when initing, not needed
        init(ser)

    strips = [0, strips[0]-1, strips[1]-1]
    strips[0] = max(strips)

    if channel == 0:
        channels = [1, 2]
    else:
        channels = [channel]

    for channela in channels:
        command = []
        command.append(75)
        command.append(channela)
        command.append(modes[mode])
        command.append(direction << 4 | option << 3 | strips[channela])
        command.append(0 << 5 | group << 3 | speed)
        for color in colors:
            command.append(int(color[2:4], 16))
            command.append(int(color[:2], 16))
            command.append(int(color[4:], 16))
        for z in range(40-len(colors)):
            command.append(0)
            command.append(0)
            command.append(0)
        command = bytearray(command)
        channel_commands.append([command])
    return channel_commands


def create_command(ser, channel, colors, mode, direction, option, group, speed):
    init(ser)

    commands = []
    channel_commands = []
    modes = {
        "fixed": 0,
        "breathing": 7,
        "fading": 1,
        "marquee": 3,
        "cover_marquee": 4,
        "pulse": 6,
        "spectrum": 2,
        "alternating": 5,
        "candlelight": 9,
        "wings": 12,
        "wave": 13,
        "alert": 8
    }

    strips = [0, strips_info(ser, 1)-1, strips_info(ser, 2)-1]
    strips[0] = max(strips)

    if channel == 0:
        channels = [1, 2]
    else:
        channels = [channel]

    for channela in channels:
        commands = []
        for i, color in enumerate(colors):
            command = []
            command.append(75)
            command.append(channela)
            command.append(modes[mode])
            command.append(direction << 4 | option << 3 | strips[channela])
            command.append(i << 5 | group << 3 | speed)
            for z in range(40):
                command.append(int(color[2:4], 16))
                command.append(int(color[:2], 16))
                command.append(int(color[4:], 16))
            command = bytearray(command)
            commands.append(command)

        channel_commands.append(commands)
    return channel_commands


def strips_info(ser, channel):
    time.sleep(0.2)
    out = bytearray.fromhex("8D0" + str(channel))
    ser.write(out)
    time.sleep(1)
    out = ser.read(ser.in_waiting).hex()
    if out:
        r = int(out[-1])
    else:
        r = -1
    if r <= 0:
        r = 1
    return r


def init(ser):
    #C0(ser)
    # Took out bytearray([70, 0, 192, 0, 0, 0, 255])
    initial = [bytearray.fromhex("4B" + "00"*124)]
    for array in initial:
        ser.write(array)
        time.sleep(0.2)
        ser.read(ser.in_waiting)

def C0(ser):
    while True:
        ser.write(bytearray.fromhex("C0"))
        if ser.in_waiting != 0:
            ser.read()
            break

def write(ser, outputs):
    for channel in outputs:
        for line in channel:
            if line:
                ser.write(line)
                ser.read()


def fixed(ser, gui, channel, color):

    if gui != 0:
        color = picker.pick("Color")

    command = create_command(ser, channel, [color], "fixed", 0, 0, 0, 2)
    outputs = previous.get_colors(channel, command)
    write(ser, outputs)


def breathing(ser, gui, channel, color, speed):

    if 1 <= gui <= 8:
        color = []
        for i in range(gui):
            color.append(picker.pick("Color "+str(i+1) + " of "+str(gui)))

    command = create_command(ser, channel, color, "breathing", 0, 0, 0, speed)

    outputs = previous.get_colors(channel, command)
    write(ser, outputs)


def fading(ser, gui, channel, color, speed):

    if 1 <= gui <= 8:
        color = []
        for i in range(gui):
            color.append(picker.pick("Color "+str(i+1) + " of "+str(gui)))

    command = create_command(ser, channel, color, "fading", 0, 0, 0, speed)

    outputs = previous.get_colors(channel, command)
    write(ser, outputs)


def marquee(ser, gui, channel, color, speed, size, direction):

    if gui != 0:
        gui = 1
        for i in range(1):
            color = picker.pick("Color "+str(i+1) + " of "+str(gui))

    command = create_command(ser, channel, [color], "marquee", direction, 0, size, speed)
    outputs = previous.get_colors(channel, command)
    write(ser, outputs)


def cover_marquee(ser, gui, channel, color, speed, direction):

    if 1 <= gui <= 8:
        color = []
        for i in range(gui):
            color.append(picker.pick("Color "+str(i+1) + " of "+str(gui)))

    command = create_command(ser, channel, color, "cover_marquee", direction, 0, 0, speed)
    outputs = previous.get_colors(channel, command)
    write(ser, outputs)


def pulse(ser, gui, channel, color, speed):

    if 1 <= gui <= 8:
        color = []
        for i in range(gui):
            color.append(picker.pick("Color "+str(i+1) + " of "+str(gui)))

    command = create_command(ser, channel, color, "pulse", 0, 0, 0, speed)
    outputs = previous.get_colors(channel, command)
    write(ser, outputs)


def spectrum(ser, channel, speed, direction):

    command = create_command(ser, channel, ["0000FF"], "spectrum", direction, 0, 0, speed)

    outputs = previous.get_colors(channel, command)
    write(ser, outputs)


def alternating(ser, gui, channel, color, speed, size, moving, direction):

    if gui != 0:
        color = []
        gui = 2
        for i in range(2):
            color.append(picker.pick("Color "+str(i+1) + " of "+str(gui)))

    if moving:
        option = 1
    else:
        option = 0

    command = create_command(ser, channel, color, "alternating", direction, option, size, speed)
    outputs = previous.get_colors(channel, command)
    write(ser, outputs)


def candlelight(ser, gui, channel, color):

    if gui != 0:
        color = picker.pick("Color")

    command = create_command(ser, channel, [color], "candlelight", 0, 0, 0, 0)

    outputs = previous.get_colors(channel, command)
    write(ser, outputs)


def wings(ser, gui, channel, color, speed):

    if gui != 0:
        color = picker.pick("Color")

    command = create_command(ser, channel, [color], "wings", 0, 0, 0, speed)
    outputs = previous.get_colors(channel, command)
    write(ser, outputs)


def power(ser, channel, state):
    if state.lower() == 'on':
        fixed(ser, 0, channel, "FFFFFF")
    elif state.lower() == 'off':
        fixed(ser, 0, channel, "000000")
    else:
        raise InvalidCommand("No such power state")

def custom(ser, gui, channel, colors, mode, speed):
    strips = [strips_info(ser, 1), strips_info(ser, 2)]

    if mode not in ['fixed', 'breathing', 'wave']:
        raise InvalidCommand("No such mode for custom")

    if 1 <= gui <= 40:
        colors = []
        for i in range(gui):
            colors.append(picker.pick("Color "+str(i+1) + " of "+str(gui)))

    command = create_custom(ser, channel, colors, mode, 0, 0, 0, speed, strips)
    outputs = previous.get_colors(channel, command)
    write(ser, outputs)


def profile_add(name):
    previous.add_profile(name)

def profile_rm(name):
    previous.rm_profile(name)

def profile_list():
    print(previous.list_profile())
    return previous.list_profile()

def profile_apply(ser, name):
    commands = previous.apply_profile(name)
    init(ser)
    write(ser, commands)

def write_previous(ser):
    write(ser, previous.get_previous())

if __name__ == '__main__':
    main()
